function TraceBode(varargin)
% Trace le diagramme de bode assymptotique & rel
% des systmes passs en paramtres
% Syntaxe : TraceBode(sys1,'r',sys2,'b',...,Ep1,Ep2)
%    Ep1 : epaisseur de trac des assymptotes
%    Ep2 : epaisseur de trac de la courbe rele

% Calcuyl du nombre de systme  reprsenter
NbSystemes=(length(varargin)-2)/2;

figure;
Ep1=varargin{length(varargin)-1};
Ep2=varargin{length(varargin)};
assignin('base','test',varargin);

% Recherche des bornes min & max pour tous les systmes
for kk=1:NbSystemes
   sys=varargin{2*(kk-1)+1};
   Coul=varargin{2*(kk-1)+2};
   [z,p,k]=zpkdata(sys,'v');
   GainStat=k;

   % Mmorisation des donnes
   TabSys{kk}=sys;
   TabCoul{kk}=Coul;
   
   % Recherche des intgrateurs
   indices=find(p==0);
   NbInte=length(indices);
   % Construction du tableau contenant les poles du systme
   pp=[];
   for i=1:length(p)
      if p(i)~=0
         pp=[pp p(i)];
      end;
   end;
   TabPoles{kk}=pp;
   TabNbInte{kk}=NbInte;
   
   % Recherche des drivateurs
   indices=find(z==0);
   NbDeriv=length(indices);
   % Construction du tableau contenant les zros du systme
   zz=[];
   for i=1:length(z)
      if z(i)~=0
         zz=[zz z(i)];
      end;
   end;
   TabZeros{kk}=zz;
   TabNbDeriv{kk}=NbDeriv;
   
   % Calcul du gain statique du systme
   GainStat=GainStat/prod(-pp);
   GainStat=GainStat*prod(-zz);
   TabGainStatique{kk}=GainStat;
   
   % Recherche des bornes
   if (~isempty(pp))&(isempty(zz))
       % Que des ples
       Wmin=min(abs(pp));
       Wmax=max(abs(pp));
   elseif (~isempty(zz))&(isempty(pp)) 
       % Que des zros
       Wmin=min(abs(zz));
       Wmax=max(abs(zz));
   elseif (isempty(zz))&(isempty(pp)) 
       % Que des intgrateurs ou des gains
       Wmin=1;
       Wmax=1;
   else
       % Ples et zros
       Wmin=min(abs(max(zz)),abs(max(pp)));
       Wmax=max(abs(min(zz)),abs(min(pp)));
   end;
    
   % Arrondis  la puissance de 10
   WWmin(kk)=10^(floor(log10(Wmin))-2);
   WWmax(kk)=10^(ceil(log10(Wmax))+2);
end; % Fin du for kk=1:NbSystemes


Wmin=min(WWmin);
Wmax=max(WWmax);

for kk=1:NbSystemes
   pp=TabPoles{kk};
   zz=TabZeros{kk};
   sys=TabSys{kk};
   NbDeriv=TabNbDeriv{kk};
   NbInte=TabNbInte{kk};
   
   GainStatique=TabGainStatique{kk};
   Coul=TabCoul{kk};
   
   % ======================================================
   % Calcul de la courbe de gain assymptotique
   % ======================================================
   PenteGain=-NbInte+NbDeriv;
   FreqGain=[Wmin Wmax];
   
   % Prise en compte des ples
   for i=1:length(pp)
      j=1;
      while (FreqGain(j)<abs(pp(i)))&(j<length(FreqGain)) 
         j=j+1;
      end;
      
      FreqGain=[FreqGain(1:j-1) abs(pp(i)) FreqGain(j:length(FreqGain))];
      PenteGain=[PenteGain(1:j-1) PenteGain(j-1)-1 PenteGain(j:length(PenteGain))-1];
   end;
   
   % Prise en compte des zros
   for i=1:length(zz)
      j=1;
      while (FreqGain(j)<abs(zz(i)))&(j<length(FreqGain)) 
         j=j+1;
      end;
      FreqGain=[FreqGain(1:j-1) abs(zz(i)) FreqGain(j:length(FreqGain))];
      PenteGain=[PenteGain(1:j-1) PenteGain(j-1)+1 PenteGain(j:length(PenteGain))+1];
   end;
   
   
   % ======================================================
   % Calcul de la courbe de phase assymptotique
   % ======================================================
   PentePhase=0;
   FreqPhase=[Wmin Wmax];
   
   % Prise en compte des ples
   for i=1:length(pp)
      j=1;
      
      while (FreqPhase(j)<abs(pp(i)/10))&(j<length(FreqPhase)) 
         j=j+1;
      end;
      
      if FreqPhase(j)~=abs(pp(i)/10)
         % Ple inexistant
         FreqPhase=[FreqPhase(1:j-1) abs(pp(i)/10) FreqPhase(j:length(FreqPhase))];
         
         if pp(i)<0
            % Pole stable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)-1 PentePhase(j:length(PentePhase))-1]; 
         else
            % Pole instable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)+1 PentePhase(j:length(PentePhase))-1]; 
         end;
      else
         % Ple multiple : on ne rajoute pas de nouvelle cassure, on ne fait qu'augmenter la pente
         if pp(i)<0
            % Pole stable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))-1]; 
         else
            % Pole instable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))-1]; 
         end;
      end;
      
      
      j=1;
      while (FreqPhase(j)<abs(pp(i)*10))&(j<length(FreqPhase)) 
         j=j+1;
      end;
      if FreqPhase(j)~=abs(pp(i)*10)
         % Ple inexistant
         
         FreqPhase=[FreqPhase(1:j-1) 10*abs(pp(i)) FreqPhase(j:length(FreqPhase))];
         if pp(i)<0
            % Pole stable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)+1 PentePhase(j:length(PentePhase))+1]; 
         else
            % Pole instable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)-1 PentePhase(j:length(PentePhase))-1]; 
         end;
      else
         % Ple multiple : on ne rajoute pas de nouvelle cassure, on ne fait qu'augmenter la pente
         if pp(i)<0
            % Pole stable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))+1]; 
         else
            % Pole instable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))-1]; 
         end;
         
      end;
   end;
   
   % Prise en compte des zros
   for i=1:length(zz)
      j=1;
      while (FreqPhase(j)<abs(zz(i)/10))&(j<length(FreqPhase)) 
         j=j+1;
      end;
      if FreqPhase(j)~=abs(zz(i)/10)
         % Zro inexistant
         FreqPhase=[FreqPhase(1:j-1) abs(zz(i)/10) FreqPhase(j:length(FreqPhase))];
         if zz(i)<0
            % Zero stable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)+1 PentePhase(j:length(PentePhase))+1]; 
         else
            % Zero instable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)-1 PentePhase(j:length(PentePhase))-1]; 
         end;
      else
         if zz(i)<0
            % Zero stable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))+1]; 
         else
            % Zero instable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))-1]; 
         end;
      end;
      
      j=1;
      while (FreqPhase(j)<abs(zz(i)*10))&(j<length(FreqPhase)) 
         j=j+1;
      end;
      if FreqPhase(j)~=abs(zz(i)*10)
         % Zro inexistant
         FreqPhase=[FreqPhase(1:j-1) 10*abs(zz(i)) FreqPhase(j:length(FreqPhase))];
         if zz(i)<0
            % Zero stable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)-1 PentePhase(j:length(PentePhase))-1]; 
         else
            % Zero instable
            PentePhase=[PentePhase(1:j-1) PentePhase(j-1)+1 PentePhase(j:length(PentePhase))+1]; 
         end;
      else
         % Zro multiple : on ne rajoute pas de nouvelle cassure, on ne fait qu'augmenter la pente
         if zz(i)<0
            % Zero stable
            PentePhase=[PentePhase(1:j-1) PentePhase(j:length(PentePhase))-1]; 
         else
            % Zero instable
            PentePhase=[PentePhase(1:j-1)  PentePhase(j:length(PentePhase))+1]; 
         end;
      end;
   end;
   % ======================================================
   % Calcul des assymptotes
   % ======================================================
         
   % Initialisation : prise en compte des intgrateurs
   Xgain=Wmin;
   Ygain=-20*(NbInte-NbDeriv)*log10(Wmin)+20*log10(abs(GainStatique));

   for i=1:length(FreqGain)-1
      Xgain(i+1)=FreqGain(i+1);
      if PenteGain(i)~=0
          Ygain(i+1)=Ygain(i)+20*PenteGain(i)*log10(Xgain(i+1)/Xgain(i));
      else
          Ygain(i+1)=Ygain(i);
      end;
   end;
   
   Xphase=Wmin;
   Yphase=-90*(NbInte-NbDeriv)+180*(GainStatique<0);
   
   for i=1:length(FreqPhase)-1
      Xphase(i+1)=FreqPhase(i+1);
      Yphase(i+1)=Yphase(i)+45*PentePhase(i)*log10(Xphase(i+1)/Xphase(i));
   end;
   
   % Calcul de la courbe relle
   WW=logspace(log10(Wmin),log10(Wmax),500);
   [MAG,PHASE,W]=bode(sys,WW);
   
   Mbo=20*log10(squeeze(MAG));
   Pbo=squeeze(PHASE);
   Fbo=W;
   
   subplot(2,1,1);
   semilogx(Xgain,Ygain,Coul,'linewidth',Ep1);
   hold on;
   semilogx(Fbo,Mbo,Coul,'linewidth',Ep2);
   grid on;
   xlabel('\omega(rd/s)');
   ylabel('Gain (dB)');
   
   subplot(2,1,2);
   semilogx(Xphase,Yphase,Coul,'linewidth',Ep1);
   hold on;
   semilogx(Fbo,Pbo,Coul,'linewidth',Ep2);
   grid on;
   xlabel('\omega(rd/s)');
   ylabel('Phase ()');
   
end; % Fin du for kk=1:NbSystemes

